// file: $USFS/class/image_analysis/v6.1/imaga_dct_04.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "image_analysis.h"
#include "image_analysis_constants.h"

// method: dct_trans_cc
//
// arguments: none
//
// return: a logical_1 value indicating status
//
// this method computes the DCT on the image block and stores the coefficients
// in the same buffer
//
logical_1 Image_analysis::dct_trans_cc() {
  
  // create temporary space for the DCT coefficients
  //
  float_4* dct_r = new float_4[dct_blk_size_d * dct_blk_size_d];
  float_4* dct_g = new float_4[dct_blk_size_d * dct_blk_size_d];
  float_4* dct_b = new float_4[dct_blk_size_d * dct_blk_size_d];
  
  if (dct_r == NULL) {
    error_handler_cc((char_1*)"dct_trans_cc",
		     (char_1*)"error allocating memory");
    return ISIP_FALSE;
  }

  if (dct_g == NULL) {
    error_handler_cc((char_1*)"dct_trans_cc",
		     (char_1*)"error allocating memory");
    return ISIP_FALSE;
  }
  
  if (dct_b == NULL) {
    error_handler_cc((char_1*)"dct_trans_cc",
		     (char_1*)"error allocating memory");
    return ISIP_FALSE;
  }

  memset(dct_r, 0, dct_blk_size_d * dct_blk_size_d * sizeof(float_4));
  memset(dct_g, 0, dct_blk_size_d * dct_blk_size_d * sizeof(float_4));
  memset(dct_b, 0, dct_blk_size_d * dct_blk_size_d * sizeof(float_4));
  
  // create intermediate storage for the DCT algorithm
  //
  float_4* intermed_r = new float_4[dct_blk_size_d * dct_blk_size_d];
  float_4* intermed_g = new float_4[dct_blk_size_d * dct_blk_size_d];
  float_4* intermed_b = new float_4[dct_blk_size_d * dct_blk_size_d];
  
  if (intermed_r == NULL) {
    error_handler_cc((char_1*)"dct_trans_cc",
		     (char_1*)"error allocating memory");
    return ISIP_FALSE;
  }

  if (intermed_g == NULL) {
    error_handler_cc((char_1*)"dct_trans_cc",
		     (char_1*)"error allocating memory");
    return ISIP_FALSE;
  }
  
  if (intermed_b == NULL) {
    error_handler_cc((char_1*)"dct_trans_cc",
		     (char_1*)"error allocating memory");
    return ISIP_FALSE;
  }

  memset(intermed_r, 0, dct_blk_size_d * dct_blk_size_d * sizeof(float_4));
  memset(intermed_g, 0, dct_blk_size_d * dct_blk_size_d * sizeof(float_4));
  memset(intermed_b, 0, dct_blk_size_d * dct_blk_size_d * sizeof(float_4));

  // compute the intermediate constants
  //
  for (int_4 i=0; i<dct_blk_size_d; i++) {
    for (int_4 j=0; j<dct_blk_size_d; j++) {
      intermed_r[i * dct_blk_size_d + j] = 0;
      intermed_g[i * dct_blk_size_d + j] = 0;
      intermed_b[i * dct_blk_size_d + j] = 0;
      for (int_4 k=0; k<dct_blk_size_d; k++) {
	intermed_r[i * dct_blk_size_d + j] += dct_r_red_d[i * dct_blk_size_d +
	  k] * dct_r_cos_d[k][j];
	intermed_g[i * dct_blk_size_d + j] += dct_r_grn_d[i * dct_blk_size_d +
	  k] * dct_r_cos_d[k][j];
	intermed_b[i * dct_blk_size_d + j] += dct_r_blu_d[i * dct_blk_size_d +
	  k] * dct_r_cos_d[k][j];
      }
    }
  }

  // compute the DCT coefficients one by one
  //
  float_4 C1 = 1.0;
  float_4 C2 = 1.0;
  for (int_4 u=0; u<dct_blk_size_d; u++) {
    for (int_4 v=0; v<dct_blk_size_d; v++) {
      dct_r[u * dct_blk_size_d + v] = 0;
      dct_g[u * dct_blk_size_d + v] = 0;
      dct_b[u * dct_blk_size_d + v] = 0;
      C1 = 1.0;
      C2 = 1.0;
      for (int_4 j=0; j<dct_blk_size_d; j++) {
	dct_r[u * dct_blk_size_d + v] += intermed_r[j * dct_blk_size_d + v]
	  * dct_r_cos_d[j][u];
	dct_g[u * dct_blk_size_d + v] += intermed_g[j * dct_blk_size_d + v]
	  * dct_r_cos_d[j][u];
	dct_b[u * dct_blk_size_d + v] += intermed_b[j * dct_blk_size_d + v]
	  * dct_r_cos_d[j][u];
      }
      if (u == 0) { C1 = 0.707106781; }
      if (v == 0) { C2 = 0.707106781; }
      dct_r[u * dct_blk_size_d + v] *= (4 * C1 * C2 / dct_blk_size_d
	/ dct_blk_size_d);
      dct_g[u * dct_blk_size_d + v] *= (4 * C1 * C2 / dct_blk_size_d
	/ dct_blk_size_d);
      dct_b[u * dct_blk_size_d + v] *= (4 * C1 * C2 / dct_blk_size_d
	/ dct_blk_size_d);
    }
  }
  
  for (int_4 i=0; i<(dct_blk_size_d * dct_blk_size_d); i++) {
    dct_r_red_d[i] = dct_r[i];
    dct_r_grn_d[i] = dct_g[i];
    dct_r_blu_d[i] = dct_b[i];
  }

  // free memory
  //
  delete [] intermed_r;
  delete [] intermed_g;
  delete [] intermed_b;
  delete [] dct_r;
  delete [] dct_g;
  delete [] dct_b;

  // exit gracefully
  //
  return ISIP_TRUE;
}
